/* This are modified Nanfor FT_*() functions for FlagShip 7 and later
 * Copyright 2012 by multisoft Datentechnik, Germany
 * Public domain for nanfor library used by FlagShip
 *
 * compile by supplied Makefile.*
 */
 
/*
 * File......: COLOR2N.C
 * Author....: David Richardson
 * CIS ID....: 72271,53
 *
 * This function is an original work by David Richardson and is placed in the
 * public domain.
 *
 * Modification for FlagShip on 2/2012 by multisoft Datentechnik
 *
 * Modification history:
 * ---------------------
 *
 *    Rev 1.0   01 Jan 1995 03:01:00   TED
 * Initial release
 *
 */


/*****************************************************************************
 *  $DOC$
 *  $FUNCNAME$
 *     FT_COLOR2N()
 *  $CATEGORY$
 *     String
 *  $ONELINER$
 *     Returns the numeric complement of a Clipper color string
 *  $SYNTAX$
 *     FT_COLOR2N( <cColor> ) -> nValue
 *  $ARGUMENTS$
 *     <cColor> is a Clipper color string
 *  $RETURNS$
 *     The numeric complement of a color string or 0 if passed color
 *     is invalid.
 *  $DESCRIPTION$
 *     This function is useful when calling other functions that expect
 *     a numeric color parameter.  It is often more convenient to pass
 *     a converted color string than having to calculate or look up the
 *     corresponding number.
 *  $EXAMPLES$
 *     nColor := FT_COLOR2N( "gr+/b" )         // returns 30
 *
 *     FT_SETATTR( 0, 0, 10, 10, nColor )
 *  $SEEALSO$
 *     FT_N2COLOR()
 *  $END$
 */
#ifdef FlagShip
  #include "FSopenc.h"
  #include "FSextend.h"
#else
  #include "EXTEND.API"
#endif

static int _ftColor2I( char * cColor );
static int _ftGetColorNum( char * cColor );
static char * _ftStripIt( char * cColor );

/*****************************************************************************
 *  nColor := FT_Color2N(cColor)
 */
#ifdef FlagShip
FSudfname(ft_color2n)
#else
CLIPPER FT_Color2N( void )
#endif
{
    int iRet = 0;
#ifdef FlagShip
    FSinit();
#endif

    // make sure parameter is a char type and that it is 8 chars or less

    if ( _parinfo( 1 ) == 1 && _parclen( 1 ) < 8 )
       iRet = _ftColor2I( _parc( 1 ) );

    _retni( iRet );

#ifdef FlagShip
    FSreturn;
#else
    return;
#endif
}


// 
// Function  :  _ftColor2I
// Purpose   :  Converts an Xbase color string to an int
// Parameters:  cColor  -  a pointer to the color string
// Returns   :  int complement of color string, or 0 if string is invalid
// 

static int _ftColor2I( char * cColor )
{
//  char * cFore = "         ", * cBack = "      ";
    char *ptr, cFore[9], cBack[9];
    unsigned int iBlink = 0, iIntense = 0, iBack = 0, i = 0;

    // copy the Clipper string to buffer, check for attributes, and
    // make lower case

//  while ( ( cFore[ i ] = cColor[ i ] ) != NULL )
    while (i < 9 && ( cFore[ i ] = cColor[ i ] ) != 0 )
    {
       // check for a blink attrib

       if ( cFore[ i ] == '*' && iBlink == 0 )
          iBlink = 128;

       // check for an intensity attrib

       if ( cFore[ i ] == '+' && iIntense == 0 )
          iIntense = 8;

        // make sure all chars are lower case

       if ( 91 > cFore[ i ] && cFore[ i ] > 64 )
          cFore[ i ] += 32;

       i++;
    }

    // check for the background color

//  while ( cColor[ iBack++ ] != '/' && cColor[ iBack ] != NULL );
    while (iBack < 9 &&  cColor[ iBack++ ] != '/' && cColor[ iBack ] != 0 );

    if ( cColor[--iBack ] == '/' )
    {
//     cBack = cFore + iBack + 1;
       ptr = cFore + iBack + 1;
//     cFore[ iBack ] = NULL;
       cFore[ iBack ] = 0;
    }

    // calculate and return the value

//    return ( iIntense + iBlink + _ftGetColorNum( _ftStripIt( cFore ) ) +
//                          ( 16 * _ftGetColorNum( _ftStripIt( cBack ) ) ) );
    return ( iIntense + iBlink + _ftGetColorNum( _ftStripIt( cFore ) ) +
                          ( 16 * _ftGetColorNum( _ftStripIt( ptr ) ) ) );

}


// 
// Function  :  _ftGetColorNum
// Purpose   :  Returns the corresponding number for an Xbase color
// Parameters:  cColor  -  a pointer to the color string
// Returns   :  int complement of a single color
// 

static int _ftGetColorNum( char * cColor )
{
   unsigned * iColor = ( unsigned * ) cColor;

   if ( cColor[ 1 ] == 0 ) switch ( cColor[ 0 ] )
   {
         case 'n' : *iColor = 0; break;
         case 'b' : *iColor = 1; break;
         case 'g' : *iColor = 2; break;
         case 'r' : *iColor = 4; break;
         case 'w' : *iColor = 7; break;
   }
   else
   {
      if ( ( cColor[ 0 ] == 'b' ) && cColor[ 1 ] == 'g' ) *iColor = 3;
      if ( ( cColor[ 0 ] == 'r' ) && cColor[ 1 ] == 'b' ) *iColor = 5;
      if ( ( cColor[ 0 ] == 'g' ) && cColor[ 1 ] == 'r' ) *iColor = 6;
   }

   return *iColor;
}



// 
// Function  :  _ftStripIt
// Purpose   :  Removes the intensity/blink chars from the passed string
// Parameters:  cColor  -  a pointer to the color string
// Returns   :  a pointer to the modified color string
// 

static char * _ftStripIt( char * cColor )
{
   unsigned i = 0;

   // move past any leading markers

   while ( *cColor == '+' || *cColor == '*' ) cColor++;

   // truncate any trailing markers

   while ( cColor[ i ] && cColor[ i ] != '+' && cColor[ i ] != '*' ) i++;

   // null terminate the string

   cColor[ i ] = 0;

   return cColor;
}