/*
 * File......: RestSets.Prg
 * Author....: David Husnian
 * CIS ID....: ?
 *
 * This is an original work by David Husnian and is placed in the
 * public domain.
 *
 * Extended 2012 by multisoft to restore SET KEY and SET FUNCTION as well
 * (extension available with FS2 Toolbox)
 *
 * Modification history:
 * ---------------------
 *
 *    Rev 1.2   15 Aug 1991 23:02:34   GLENN
 * Forest Belt proofread/edited/cleaned up doc
 *
 *    Rev 1.1   27 May 1991 13:04:20   GLENN
 * Minor documentation change.
 *
 *    Rev 1.0   01 Apr 1991 01:02:06   GLENN
 * Nanforum Toolkit
 *
 */


/*  $DOC$
 *  $FUNCNAME$
 *     FT_RESTSETS()
 *  $CATEGORY$
 *     Environment
 *  $ONELINER$
 *     Restore status of all SET command settings
 *  $SYNTAX$
 *     FT_RESTSETS( [ <aOldSets> ] ) -> NIL
 *  $ARGUMENTS$
 *     aOldSets is an array of SET settings created by FT_SAVESETS()
 *  $RETURNS$
 *     NIL
 *  $DESCRIPTION$
 *     This function "restores" the SET Settings, i.e., it sets them to the
 *     values in the array aOldSets.  The following SETs are not currently
 *     supported: FILTER, FORMAT, FUNCTION, INDEX, KEYS, MODE, ORDER,
 *     PROCEDURE, RELATION, TYPEAHEAD
 *     FlagShip: SET KEY, SET FUNCTION are supported with FS2 Toolbox
 *  $EXAMPLES$
 *     FT_RESTSETS(aOldSets)
 *  $INCLUDE$
 *     SET.CH
 *  $SEEALSO$
 *     FT_SAVESETS() FT_SETCENTURY()
 *  $END$
 */


#ifndef FlagShip
#include "set.ch"
#endif

#Define FT_EXTRA_SETS    2
#DEFINE FT_SET_CENTURY   _SET_COUNT + 1
#DEFINE FT_SET_BLINK     _SET_COUNT + 2

FUNCTION  FT_RESTSETS(aOldSets)
   local ii, aSet := {}, aSetKeys := {}, aFunKeys := {}

   for ii := 1 to len(aOldSets)
      if valtype(aOldSets[ii]) == "A" .and. valtype(aOldSets[ii,1]) == "C" .and. aOldSets[ii,1] == "SETKEY" 
          aadd(aSetKeys, aOldSets[ii,2]) 
      elseif valtype(aOldSets[ii]) == "A" .and. valtype(aOldSets[ii,1]) == "C" .and. aOldSets[ii,1] == "FUNKEY" 
          aadd(aFunKeys, aOldSets[ii,2]) 
      else
          aadd(aSet, aOldSets[ii])
      endif
   next
  
   AEVAL(aSet, ;
         { | xElement, nElementNo | ;
           SET(nElementNo, xElement) }, ;
         1, _SET_COUNT )

   FT_SETCENTURY(aSet[FT_SET_CENTURY])
   SETBLINK(aSet[FT_SET_BLINK])

#ifdef VFS_FS2TOOLS  
   if len(aSetKeys) > 0
      RestSetKey(aSetKeys)       // restore SET KEY...
   endif
   for ii := 1 to len(aFunKeys)  // restore SET FUNCTION...
      if valtype(aFunKeys[ii]) == "A" .and. len(aFunKeys[ii]) >= 2 .and. valtype(aFunKeys[ii,1]) == "N"
         if valtype(aFunKeys[ii,2]) == "C"
            SET FUNCTION (aFunKeys[ii,1]) TO (aFunKeys[ii,2])
         else
            SET FUNCTION (aFunKeys[ii,1]) TO 
         endif
      endif
   next
#endif

   RETURN (NIL)                         // FT_RestSets
